/***********************************************************************
 *                                                                     *
 * Dateiname         :  HF_Simulator.ino                               *
 *                                                                     *
 * Projekt,Version   :  Version 0.1                                    *
 *                                                                     *
 *                                                                     *
 * Compiler-Version  : Arduino 1.6.5r2                                 *
 *                                                                     *
 * Prozessor         : ATMega328                                       *
 *                                                                     *
 * Unterprogramme    :                                                 *
 *                                                                     *
 *                                                                     *
 * Kurzbeschreibung  : Das Programm dient zur Simulation des M-Link    *
 *                     HF-Moduls in der MPX Royal EVO. Mit dieser      *
 *                     Simulation ist es dann moeglich andere HF-Module*
 *                     intern an die EVO anzuschliessen, damit der     *
 *                     L/S-Stecker wieder frei bleibt. Ebenfalls wird  *
 *                     es moeglich fremde Telemetriedaten im MPX-      *
 *                     Sensorformat im Sededisplay anzeigen zu lassen. *
 *                     Der Prozessor lauft in der EVO mit 3V3 und 8MHz.*
 *                                                                     *
 *                                                                     *
 *                                                                     *
 *                                                                     *
 *                                                                     *
 * zus. Dokument.    : Strucktogramm                                   *
 *                                                                     *
 * Hinweise          : Timer 0 wird von Arduino verwendet und darf     *
 *                     nicht benutzt werden!                           *
 *                                                                     *
 *                                                                     *
 * Ersteller,Datum   :  D. Kammerer, den 26.-01.-2016                  *
 *                                                                     *
 * geaendert         :                                                 *
 *                                                                     *
 * Unterschiede zur                                                    *
 * version           :                                                 *
 *                                                                     *
 *                                                                     *
 ***********************************************************************/

//*******************************************************
//** Hier alle benoetigten Header-Files mit einbinden  **
//*******************************************************


//********************************************************
//** Hier alle allgemeinen Globalen Variablen einbinden **
//********************************************************
byte Ausl_Taste = 9;       	// Mit welcher Taste wir eine Aktion ausgeloest?
							// Es sind nur die Kanaele 9, 10, 11 und 12 erlaubt(meine Festlegung).
							// ->  9 --> Ausloesung mit Kanal  9 (Bit 0 in Byte Schaltkanal) 
							// -> 10 --> Ausloesung mit Kanal  9 (Bit 1 in Byte Schaltkanal) 
							// -> 11 --> Ausloesung mit Kanal 11 (Bit 2 in Byte Schaltkanal)
							// -> 12 --> Ausloesung mit Kanal 12 (Bit 3 in Byte Schaltkanal)                           // Wird aus dem EEPROM gelesen.

byte  EVO_Taste_Status = 0; // Verwendet, um den Schatkanal richtig zu interpretieren

byte Schaltkanal;  

// = 6 --> kein Telemetrieempfaenger vorhanden bzw. 
// Telemetrieempfaenger ausserhalb der Reichweite
// = 22 --> Telemetrieempfaenger vorhanden
byte Tel_Met_RX_da = 22; 

boolean Alarm;		// Wird TRUE, wenn ein Alarm detektiert wird.

// ASCII-Zeichen der Firmwareversion des HF-Modules
byte Modul_Version[4];     

/************************************************************************
/ ** Auswerten der seriel empangenen Daten vom COM-Port des HF-Modules **
/ ** Achtung:                                                          **
/ ** Eine 0x02 zeigt den Start eines Frames an.                        **
/ ** Eine 0x03 zeigt das Ende  eines Frames an.                        **
/ ** Kommen in den Daten eine 0x02 oder eine 0x03 vor, so wird die     **
/ ** Zahl wie folgt umgewandelt:                                       **
/ ** Es wird eine 0x1b (ESC) vorangestellt. Zum eigentlichen Wert      **
/ ** wird dann 0x20 hinzu addiert.                                     **
/ ** Z.B. soll der Wert 3 uebertragen werden so werden folgende        **
/ ** 2Bytes uebertragen --> 0x1b 0x23                                  **
  ***********************************************************************/
		  
void MSB_mitschreiben() {
byte Sensor_Adresse; // Anzeigeadresse des Sensors
byte temp;           // Zwischenspeicher
int  Sensor_temp;    // Zwischenspeicher Sensorwert
int  Sensor_temp1;    // Zwischenspeicher Sensorwert

//  do {
    
    switch (MSB_Lese_Status){
        case 0:  
              // Grundstatus, warten auf ein Zeichen
              // Hier wird mit Serial.available nachgeschaut, ob ein Zeichen 
              // ueber den COM-Port des Modules angekommen ist. 
			  // Ist kein Zeichen gekommen, wird in diesem Zustand 0 verblieben,
              //  aber es wird nicht in einer Endlosschleife gewartet!
              // Ist ein Zeichen angekommen, dann kommen auch gleich weitere 
              // Zeichen ueber den COM-Port. Dieses erste Zeichen wird gelesen und es 
              // wird in den Status restliche Zeichen Lesen gewechselt.
              if (Serial2.available() > 0) { // Sind Zeichen ueber dem COM-Port angekommen?
                 MSB_Lese_Status = 1;       // Ja, dann in den naechsten Status wechseln
              }
              else {
                 MSB_Lese_Status = 0;  // Kein Zeichen gekommen? Schade, 
                                       // dann bleibe in diesem Status.
              }
         break;
         case 1:
              // Auswerten des ersten Zeichens, das empfangen wurde.
              temp = Serial2.read();  // Das erste angekommene Byte lesen

			  // Ist das gelesene Byte eine 0x02, dann haben wir den start
			  // des Frames gefunden.
              // Zur weiteren Verarbeitung wird in den Status 3 verzweigt.
              if (temp == 2) MSB_Lese_Status = 3;
			  // Ist der Start des Frames noch nicht gefunden,
			  // dann fangen wir wieder von Vorne an.
			  else MSB_Lese_Status = 0;
         break;
        case 3: 
              // Der Frame hat begonnen, so koennen wir weitere
			  // Daten vom COM-Port des HF-Moduls lesen.     
              // Als naechstes Byte kommt fuer mich ein Dummy-Byte
              temp = Read_TX();
              // Nun kommen 3 Byte mit der Firmwareversion des HF-Modules 
              Modul_Version[0] = Read_TX();
              Modul_Version[1] = Read_TX(); 
              Modul_Version[2] = Read_TX();

              // Nun kommen 2 Byte mit der Schaltkanal Infos
			  // Wir werten aber nur das Byte 5 aus, indem die Schaltkanalinfo 
			  // der Kanaele 9 - 12 stecken.
              Schaltkanal = Read_TX();
			  // Das Naechste Byte der Schaltkanalinfo werten wir nicht aus, da die 
			  // Kanaele 1 - 8 auf jedenfall der Steuerung des Modells vorbehalten bleiben
              temp        = Read_TX();
              // Nun kommt das Statusbyte, ob Telemetriedaten empfangen werden
              Tel_Met_RX_da = Read_TX();
              if (Tel_Met_RX_da < 10) Daten_Da = true;  // nur weiter machen, wenn ach Daten da sind.

              // Nun wird der erste Sensor ausgewertet
			  temp = Read_TX();
              Sensor_Adresse = temp / 16;  // Das High-Nibble traegt die Adresse
			  Sensor_Einheit[Sensor_Adresse] = temp & 0x0F; // Das Low-Nibble traegt die Einheit
              temp = Read_TX();            // LSB des Sensorwertes lesen
			  Sensor_temp = temp;
              // Das Alarmbit abspeichern. Ist es 1, dann haben wir einen 
              // Alarm bei diesem Sensor. Auch das Alarmbit = 0 wird abgespeichert
              Sensor_Alarm[Sensor_Adresse] = temp & 0x01;
			  // Ist ein Alarm gekommen, dann merken wir uns, da� einer da ist.
			  if (Sensor_Alarm[Sensor_Adresse] == 1) Alarm = true;
        // MSB des Sensorwertes lesen und an den LSB setzen
			  temp = Read_TX();
			  Sensor_temp1 = temp * 256;
              Sensor_temp += Sensor_temp1; 

              Sensor_daten[Sensor_Adresse] = Sensor_temp / 2; // --> Abspeichern 

        // Nun wird der zweite Sensor ausgewertet
			  temp = Read_TX();
              Sensor_Adresse = temp / 16;  // Das High-Nibble traegt die Adresse
              Sensor_Einheit[Sensor_Adresse] = temp & 0x0F;
              temp = Read_TX();            // LSB des Sensorwertes lesen
			  Sensor_temp = temp;		  
              // Das Alarmbit abspeichern. Ist es 1, dann haben wir einen 
              // Alarm bei diesem Sensor Auch das Alarmbit = 0 wird abgespeichert
              Sensor_Alarm[Sensor_Adresse] = temp & 0x01;
              // MSB des Sensorwertes lesen und an den LSB setzen
			  temp = Read_TX();
			  Sensor_temp1 = temp * 256;
              Sensor_temp += Sensor_temp1; 
              Sensor_daten[Sensor_Adresse] = Sensor_temp / 2; // --> Abspeichern 
        // Die naechsten 9 Bytes weis ich nicht fuer was sie sind       
			  // Wir haben nun alle bekannten Bytes des Frames ausgelesen und zum warten, bis das 
			  // Ende des Frames erreicht ist, wechseln wir in den Status 4 
			  MSB_Lese_Status = 4;	
        break;
		// Hier nun den Rest des Frames, der nicht ausgewertet wird, auslesen und wegschneissen!! 
		// Es wir solange gelesen, bis das Zeichen End Of Frame (0x03) erreicht wurde.
		case 4:
			  do 
			  {
					// Warten, bis ein Byte vom HF-Modul gesendet wurde
					do { 
					} while ( Serial2.available() == 0);
					temp = Serial2.read();  // Dann das angekommene Byte abholen
				  
			  } while (temp != 0x03);
	
              // Wir sind nun fertig mit dem Auslesen, gehen wieder in den warte Status 0
              MSB_Lese_Status = 0;	
		break;
     }

     // Nur behandeln, wenn ueberhaupt ein Telemetrieempfaenger 
	 // erkannt wird und er sich in Reichweite befindet.
     if (Tel_Met_RX_da < 20) {   	 
 				EVO_Taste();			// Abfragen, ob eine Taste betaetigt wurde
						                // Ist eine Taste betaetigt worden? --> Die Funktion 
										// EVO_Taste bringt einen Wert > als Null zurueck!
       }   

//  } while (endless == 1);  // Mache solange, bis keine Lust mehr
}


/****************************************************************************
 ** Liest das Byte "Schaltkanal" in dem die Info steckt, mit welchem Kanal **
 ** das Sprechen ausgeloest werden soll.                                   **
 ** Ist der entsprechende Kanal nicht betaetigt, gibt die Funktion 0 zu-   **
 ** rueck, ansonsten 255.                                                  **
 ** Bleibt der Kanal laenger als 5 Sekunden eingeschaltet, so wird die     **
 ** Funktion Ansage nach Zeit aktiviert (Sprach_ausl_Modus wird auf 3      **
 ** gesetzt). Dieser Modus bleibt solange aktiviert, bis der Kanal wieder  **
 ** ausgeschaltet wird (Sprach_ausl_Modus wird auf 2 gesetzt).             **
 ** In Sprach_ausl_Modus 3 muss deshalb diese Funktion auch aufgerufen     **
 ** werden, sonnst kommt man nie mehr in den Sprach_ausl_Modus 2 zurueck.  **
 ****************************************************************************/
void EVO_Taste(void) {
unsigned int temp_Wartezeit_taste;

	switch (EVO_Taste_Status) {
		case 0:
			// Nachschauen, ob der betreffende Schaltkanal betaetigt wurde
			Schaltkanal = Schaltkanal & Ausl_Taste;	
			// Ist der ausgewaehlte Schaltkanal betaetigt?
			if (Schaltkanal != 0) {
				EVO_Taste_Status = 1;		// Ja, dann in den Auswertestatus wechseln
			} else {
				EVO_Taste_Status = 0;		// Nein, dann weiter warten				
			}		
		break;
		case 1:
			// In diesem Status solange warten, bis der Schaltkanal wieder zurueckgenommen wird.
			Schaltkanal = Schaltkanal & Ausl_Taste;	
			// Ist der ausgewaehlte Schaltkanal immer noch betaetigt?
			if (Schaltkanal != 0) {
				temp_Wartezeit_taste++; // Mach irgendwas solange der Schaltlkanal betaetigt wird
			} else {
				EVO_Taste_Status = 0;	    // Nein, der ausgewaehlte Schalter wurde wieder ausgeschaltet,
			}							    // dann wieder in den Grundzustand wechseln.
		break;		
	}	
}

/****************************************************************************
/ ** Hier ein Zeichen vom In-Buffer lesen.                                 **
/ ** Ist das Byte gelesen, muss nachgesehen werden, ob die Daten eigent-   **
/ ** lich eine 0x02 oder 0x03 sein sollte.                                 **  
/ ** Ist das gelesene Zeichen aber ein 0x1b (ESC-Zeichen) dann muss ein    **
/ ** weiteres Zeichen gelesen werden.                                      **
/ ** Von diesem Zeichen muss nun 0x20 subtrahiert werden, um den eigent-   **
/ ** lichen Wert zu errechnen!                                             ** 
/ ***************************************************************************/ 
byte Read_TX() {
byte temp;
	// Warten, bis ein Byte vom HF-Modul gesendet wurde
	do { 
	 } while ( Serial2.available() == 0);
	temp = Serial2.read();  // Dann das angekommene Byte abholen

	// Ist das gelesenen Zeichen ein 0x1B (ESC-Zeichen)?
	if (temp == 0x1b)	// Ja, dann ...
	{
			// Naechstes Zeichen aus In-Buffer lesen und dann davon 0x20 abziehen
			// so haben wir dann das richtige Zeichen errechnet
			// Warten, bis ein Byte vom HF-Modul gesendet wurde
			do { 
			 } while ( Serial2.available() == 0);
			temp = Serial2.read();  // Dann das angekommene Byte abholen
			temp -= 0x20;		   // Nun nur noch 0x20 abziehen.
	} 

	// Wenn das zuerst gelesene Zeichen kein 0x1B war,
	// dann kann das es direkt verwendet werden
    return temp;           
}



 
